/**
 * @file FaspManager.h
 *
 * Aspera, Inc.
 */

#ifndef __FASP_MANAGER_H__
#define __FASP_MANAGER_H__

/// \mainpage Aspera C++ Fasp Manager API
///
///  \section introduction_sec Introduction
///  The FASP Manager library provides a high-level interface for creating new Aspera 
///  transfer jobs and monitoring transfer status and progress statistics through 
///  polling and/or callback mechanisms. 
///  This documentation specifies the FASP Manager library API and a sample program, 
///  FaspManagerSample.cpp, demonstrating how the API may be used in a C++ application 
///  for the following basic transfer cases:
///    - pushing (uploading) a single file or directory to a remote destination
///    - pulling (downloading) a single file or directory from a remote source
///    - transferring a list of files or directories.
///
///  Please consult Aspera if you have specific requirements, not covered in the 
///  sample code.
///
///  \section runloop_sec Building the FASP Manager Sample
///  The FaspManagerSample is an MSVC solution that builds a sample executable 
///  linking the FASP Manager library. The Sample provides an interactive shell  
///  demonstrating transfer control commands and displaying the transfer progress 
///  statistics and status events. 
///  Typing "help" at the prompt displays a complete listing of supported commands, including
///  displaying per file and whole job statistics, and commands to pause, resume, cancel,
///  and change the rate of an ongoing transfer.    
///
///
///  \section threadinit_sec Using the FASP Manager API
///  As illustrated by the FaspManagerSample, to use the FASP Manager API to start
///  an Aspera transfer, please use the following steps:
///  - Create a new FASP Manager object. 
///    The constructor creates and starts a singleton Manager.
///  In turn, the Manager creates a FASP Manager server socket and starts up a handler 
///  thread to listen for fasp management protocol messages from the underlying scp.aspera 
///  process
///  - Create a JobOrder, a description of the transfer session to be done, including:
///        - A Local or Remote endpoint Location specifying the login user, host ip, and pathnames 
///        to transfer;
///        - The transfer parameters such as target rate, bandwidth sharing policy, a 
///          minimum rate for adaptive rate control, the TCP and UDP ports used, and the 
///        encryption settings.
///  - Optionally create a CallbackFunctor (a Callback Function) that will be registered for 
///        receiving asynchronous callback messages reporting the transfer progress statistics and 
///        transfer status events.
///  - Call the Manager's LaunchJob method, passing in the JobOrder definition and Callback Functor 
/// reference.
///
///  During the transfer, the application may optionally call the Manager's ControlJob method to send commands to pause, 
///  resume or change the transfer speed or bandwidth sharing policy.

/* Other includes, for the sole purpose of making this a master include file */
#include "Environment.h"
#include "FaspManagerLocation.h"    
#include "FaspManagerFunctor.h"     
#include "FaspManagerException.h"   
#include "FaspManagerTypes.h"
#include "FaspManagerCommon.h"

/// C++ namespace for Aspera, Inc.
namespace Aspera
{
/// C++ Aspera Fasp Manager namespace
namespace FaspManager
{

/* Forward declarations */
class Job;

/* FASP Manager states */
/// Fasp Manager is idle 
#define FASP_MGR_IDLE       0x01  
/// Fasp Manager has started 
#define FASP_MGR_STARTED    0x02  
/// Fasp Manager has stopped 
#define FASP_MGR_STOPPED    0x03  
/// An error occured 
#define FASP_MGR_ERROR      0x04  

/* Globals */

/// An enumeration of FASP Manager control types
typedef enum mgr_ctrl_t
{
  CTRL_NONE        = 0,
  CTRL_PAUSE       = 0x01,
  CTRL_RESUME      = 0x02,
  CTRL_CANCEL      = 0x04,
  CTRL_UPDATE_RATE = 0x08,
  CTRL_UPDATE_MIN  = 0x10,
  CTRL_UPDATE_POLICY  = 0x20,
  CTRL_UPDATE = CTRL_UPDATE_RATE | CTRL_UPDATE_MIN | CTRL_UPDATE_POLICY,
} mgr_ctrl_t;

inline mgr_ctrl_t operator|(mgr_ctrl_t a, mgr_ctrl_t b)
    { return (mgr_ctrl_t)((int)a | (int)b); }
inline mgr_ctrl_t& operator|=(mgr_ctrl_t& a, mgr_ctrl_t b)
    { return (a = (a | b)); }
inline mgr_ctrl_t operator&(mgr_ctrl_t a, mgr_ctrl_t b)
    { return (mgr_ctrl_t)((int)a & (int)b); }

/// \brief A class to manage FASP jobs.
///
/// A FASP Manager is responsible for controlling (i.e., launching, pausing,
/// resuming, updating, cancelling) FASP Jobs.  The manager is also charged with
/// gathering statistics on each of these jobs.  In this context, a FASP Job is
/// a file, directory, or combination thereof to be transfered.
class Manager
{

  public:
    
    /// Gets a singleton reference to a FASP Manager.
    ///
    /// \return a reference to a FASP Manager
    FASP_MANAGER_API static Manager *GetInstance( bool owns_log = true );

    /// Stops and destructs a singleton reference to a FASP Manager.
    FASP_MANAGER_API static void Destroy( );

    /// Launches a FASP Job.
    ///
    /// \param order the transfer definition of the FASP Job
    /// \param fnctr the functor used in statistic notification callbacks, NULL
    ///  if there is no callback functor to register
    /// \return the job id associated with the launched FASP Job
    /// \throws XferException on transfer related errors
    /// \throws ValidationException if paths are not correctly set in the job order
    FASP_MANAGER_API virtual char const* LaunchJob( JobOrder *order, Functor *fnctr ) = 0;

    /// Controls a FASP Job.
    /// Possible controls are Pause, Resume, and Cancel.
    /// \param ctrl the control type
    /// \param up_prms the rate and/or policy change, if applicable
    /// \param jobid the job id associated with the FASP Job to control
    /// \throws Job_not_found_exception on job ids that cannot be mapped to
    /// FASP Jobs
    /// \throws fasp_state_exception on Jobs that cannot be removed in their
    /// current state
    /// \throws fasp_send_exception on errors relating to the sending of
    /// management messages
    FASP_MANAGER_API virtual void ControlJob( mgr_ctrl_t ctrl, UpdateParams *up_prms, char const* jobid ) = 0;

    /// Adds a source path to a persistent transfer session.
    /// \param jobid id of the job to which the source path is to be added
    /// \param src_path the new source path to be added (must be a file if using ranges)
    /// \param dst_path the destination path for the source
    /// \param start the start byte of source file. 0 indicates beginning of the file
    /// \param end the end byte of source file. 0 indicates end of the file
    /// \throws Job_not_found_exception on job ids that cannot be mapped to
    /// FASP Jobs
    /// \throws fasp_state_exception on Jobs that are either completed or not persistent
    /// \throws fasp_send_exception on errors relating to the sending of
    /// management messages
    FASP_MANAGER_API virtual void AddSourcePath( char const* jobid, char const* src_path, char const* dst_path, u_int64_t start=0, u_int64_t end=0 ) = 0;

	/// Cancels a transfer job and aborts the transfer, if still running. Results in SESSION_ERROR notification.  
    /// \param jobid id of the job to which source path is to be added
    /// \throws JobNotFoundException on job ids that cannot be mapped to
    /// FASP Jobs
    /// \throws StateException on Jobs that are either completed or not persistent
    /// \throws SendException on errors relating to the sending of
    /// management messages
    FASP_MANAGER_API virtual void CancelJob( char const* jobid ) = 0;

    /// Stops a persistent transfer job and aborts the transfer, if still running. Normally, jobs 
    /// are automatically terminated once all sources are transferrred (or failed). 
    /// The only exception is Persistent jobs which persist until you invoke this function. This 
    /// is treated as normal termination of a persistent job and no errors are raised.
    /// \param jobid id of the job to which source path is to be added
    /// \throws JobNotFoundException on job ids that cannot be mapped to
    /// FASP Jobs
    /// \throws StateException on Jobs that are either completed or not persistent
    /// \throws SendException on errors relating to the sending of
    /// management messages
    FASP_MANAGER_API virtual void StopPersistentJob( char const* jobid ) = 0;

    /// Locks a persistent transfer job and prevents it from accepting paths further.
    /// Once locked, a persistent job will terminate when it completes 
    /// transferring all paths already submitted to it, just like a regular job.
    /// \param jobid id of the job to which source path is to be added
    /// \throws JobNotFoundException on job ids that cannot be mapped to
    /// FASP Jobs
    /// \throws StateException on Jobs that are either completed or not persistent
    /// \throws SendException on errors relating to the sending of
    /// management messages
    FASP_MANAGER_API virtual void LockPersistentJob( char const* jobid ) = 0;

    /// Removes a FASP Job from the FASP Manager.
    ///
    /// \param jobid the job id associated with the FASP Job to remove
    /// \throws JobNotFoundException on job ids that cannot be mapped to
    /// FASP Jobs
    /// \throws StateException on Jobs that cannot be removed in their
    /// current state
    FASP_MANAGER_API virtual void RemoveJob( char const* jobid ) = 0;

    /// Adds a global callback that gets notified for events on job with the specified id.
    ///
    /// \param jobid the job id associated with the FASP Job to add a callback
    /// \param fnctr the FASP functor acting as the callback
    /// \throws JobNotFoundException on job ids that cannot be mapped to
    /// FASP Jobs
    FASP_MANAGER_API virtual void AddCallback( char const* jobid, Functor *fnctr ) = 0;

    ///
    /// Adds a global callback that gets notified for events on all active transfers
    ///
    /// \param fnctr the FASP functor acting as the callback
    FASP_MANAGER_API virtual void AddGlobalCallback( Functor *fnctr ) = 0;


    /// Removes a global callback
    ///
    /// \param fnctr the FASP functor acting as the callback
    FASP_MANAGER_API virtual void RemoveGlobalCallback( Functor *fnctr ) = 0;

    /// Gets a list of FASP Jobs.
    /// This list comprises a snapshot of the jobs running at the time of
    /// this call.
    ///
    /// \return pointer to a null terminated array of strings (char *'s)
    FASP_MANAGER_API virtual char const* const* GetJobIdList( ) = 0;

    /// Gets FASP Job job level stats.
    /// \param jobid the job id associated with the FASP Job
    /// \return the FASP Job stats
    /// \throws JobNotFoundException on job ids that cannot be mapped to
    /// FASP Jobs
    FASP_MANAGER_API virtual JobStats GetJobStats( char const* jobid )= 0;

    /// Once this is turned on, you can use a global callback to receive 
    /// notifications for sessions intiated outside FaspManager. This feature
    /// can be used to monitor transfers on an Aspera server. NOTE: This must 
    /// be turned on at the time an external transfer is started. Fasp Manager 
    /// cannot send notifications for transfers already running at the time 
    /// this feature is turned on.

    /// \param On the boolean parameter to enable/disable external session monitoring.
    /// \return true if the request to start (or to stop) listening succeeded
    FASP_MANAGER_API virtual bool ListenForServerSessions(bool On) = 0;

    /// Set the debug logging level for this Fasp Manager
    /// \param level the logging level 0 to 2
    FASP_MANAGER_API virtual void SetDebugLevel(int level) = 0;

  protected:

    /// Constructs this FASP Manager.
    FASP_MANAGER_API Manager( );

    /// Destructs this FASP Manager.
    FASP_MANAGER_API virtual ~Manager( );
};

} /* namespace FaspManager */
} /* namespace Aspera */

#endif /* __FASP_MANAGER_H__ */
