/**
 * @file FaspManagerLocation.h
 *
 * Aspera, Inc.
 */

#ifndef __FASP_MANAGER_LOCATION_H__
#define __FASP_MANAGER_LOCATION_H__

#include "FaspManagerTypes.h"

namespace Aspera
{
namespace FaspManager
{

/// \brief A class representing a transfer end point.
///
/// This class holds the information about the paths to 
/// source and destination files, authentication information etc.
class Location
{
  /* Member functions */
  public:

    /// Create this _local_ FASP location.
    FASP_MANAGER_API static Location * CreateLocal( );

    /// Create this _remote_ FASP location (password authentication).
    ///
    /// \param host the location hostname
    /// \param user the location username
    /// \param passwd the location password
    FASP_MANAGER_API static Location * CreateRemote( const char * host, const char* user, const char* passwd );

    /// Create this _remote_ FASP location (public key authentication).
    ///
    /// \param host the location hostname
    /// \param user the location username
    /// \param identity the path to local file containing the SSH identity (private key)
    /// \param passphrase the passphrase used to secure the private key (may be empty)
    FASP_MANAGER_API static Location * CreateRemote( const char* host, const char* user, const char* identity, const char* passphrase );

    /// Specify the range of bytes to be downloaded.
    /// Byte range is to be specified on the source Location object.
    /// This currently works for downloads only.
    /// Use -1 to indicate end of file.
    /// Throws ValidationException if used on destination object.
    ///
    /// \param start the byte position at which the transfer begins (inclusive)
    /// \param end the byte position at which the transfer ends (inclusive)
    FASP_MANAGER_API void SetByteRange(u_int64_t start, u_int64_t end);

    /// Destroys this FASP location.
    FASP_MANAGER_API void Destroy( );

    /// Copy constructs this FASP location.
    ///
    /// \param loc the FASP location to be copy constructed
    FASP_MANAGER_API Location( const Location &loc );
  
    /// Returns true iff this is a remote FASP location.
    ///
    /// \return true iff this is a remote FASP location
    FASP_MANAGER_API virtual bool IsRemote( ) const = 0;

    /// Gets the hostname associated with this _remote_ FASP location.
    ///
    /// \return the hostname associated with this FASP location,
    /// or NULL if local endpoint
    FASP_MANAGER_API virtual char* GetHostname( ) const = 0;

    /// Gets the username associated with this _remote_ FASP location.
    ///
    /// \return the username associated with this FASP location
    /// or NULL if local endpoint
    FASP_MANAGER_API virtual char* GetUsername( ) const = 0;

    /// Gets the password associated with this _remote_ FASP location.
    ///
    /// \return the password associated with this FASP location
    /// or NULL if local endpoint
    FASP_MANAGER_API virtual char* GetPassword( ) const = 0;

    /// Gets the path to the private key
    FASP_MANAGER_API virtual char* GetIdentity( ) const = 0;

    /// Adds a filepath to the list of paths associated with this FASP location.
	///
	/// Deprecated feature. Use AddFilepath(source, destination) - two argument method
    ///
    /// \param path the filepath to be added to the list
	/// \deprecated use AddFilepath(source, destination)
    FASP_MANAGER_API virtual void AddFilepath( char const* path ) = 0;

	/// Adds a filepath pair (source/destination) to the list of paths associated with this FASP location.
    ///
    /// \param path the filepath to be added to the list
    FASP_MANAGER_API virtual void AddFilepath( char const* source, char const* destination ) = 0;

  protected:
    /// Constructs this FASP location.
    FASP_MANAGER_API Location( );

    /// Destructs this FASP location.
    FASP_MANAGER_API virtual ~Location( ) = 0;

};

} /* namespace FaspManager */
} /* namespace Aspera */

#endif /* __FASP_MANAGER_LOCATION_H__ */
